<?php
namespace Teknisa\OPL\Controllers;

use \Zeedhi\Framework\DTO\Request\Row;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use Exception;
use \Teknisa\OPL\Services\Opl00800MigracaoRupturaOPLService;

class Opl00800MigracaoRupturaOPLController extends \Teknisa\GER\Controllers\TeknisaCrud {

  protected $dataSourceName = '';
  protected $crudService;

  public function __construct($crudService, $parameterBag, Opl00800MigracaoRupturaOPLService $service) {
    $this->crudService  = $crudService;
    $this->service      = $service;
    $this->parameterBag = $parameterBag;
  }

  public function alteracaoAutomaticaEntrega(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $rows = $request->getRow();
      $cdsAfs = [];

      foreach($rows['rowsUse'] as $row) {
        $adqFornecedor = $this->service->adqFornecedor($row);

        $novoCodigoAF = $this->service->procedureGeraNovoCodigo('AUTOFORNE');

        $this->service->acmInsAFMigra($row, $novoCodigoAF);

        if($adqFornecedor['IDTPFORPGTFO'] == 'V') {
          $this->service->acmInsPrazoPgtAF($row, $novoCodigoAF);
        }

        $row['NEW_NRAUTOFORN'] = $novoCodigoAF;
        $row['NEW_DTPROGENTAF'] = $rows['P_DTPROGENTAF'];

        $cdsAfs[] = $row;

        $adqPesqItmAfExistente = $this->service->adqPesqItmAfExistente($row);

        if(is_array($adqPesqItmAfExistente)) {
          $this->service->acmUpdItmAFMigra($row);
        } else {
          $this->service->acmInsItmAFMigra($row);
        }

        $this->service->acmUpdItmAfOrig($row);
      }
      
      foreach($cdsAfs as $af) {
        $adqVerifCtPag = $this->service->adqVerifCtPag($af);
        
        if($adqVerifCtPag['CONTA'] <= 1) {
          $this->service->acmUpdateTpCtPagar($af);
        }
        
        $this->service->fechamentoProcAF($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
        $this->service->excluiNFContaPagar($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
        $this->service->incluiNFContaPagar($af['CDFILDEST'], $af['CDFILLANC'], $af['NRAUTOFORN']);
        
        $adqAfOriginal = $this->service->adqAfOriginal($af);
        
        if($adqAfOriginal['ITMBAIXADO'] > 0) {
          $af['P_IDBAIXAAF'] = '1';
        } else {
          $af['P_IDBAIXAAF'] = '3';
        }
        
        $this->service->acmUpdBaixaAf($af);
        
        $this->service->AJUSTE_CONTPREVISTA($af);
      }
      
      $this->service->realizaCommit();
      
      $response->addDataSet(new DataSet('data', ['success']));
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function entregaDiretaNew(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $rows = $request->getRow();
      
      foreach($rows as $row) {
        $this->service->acmUpdItemAF1($row, 'P');
      }
      
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function entregaDiretaPaP(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $rows = $request->getRow();
      
      foreach($rows as $row) {
        $adqParasupri = $this->service->adqParasupri($row);
        
        if($adqParasupri['IDGERASOLICMPEXT'] == 'S') {
          $this->service->acmUpdItemAF($row);
        } else {
          $this->service->acmUpdItemAF1($row, 'T');
        }
      }
      
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function desfazerMigracaoItem(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $rows = $request->getRow();
      
      foreach($rows as $row) {
        $this->service->acmUpdItemAFDesfazer($row);
      }
      
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function concluirAnalise(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $row = $request->getRow();
      
      $procedure = $this->service->concluirAnalise($row);
      
      if($procedure['P_ERROS'] == 'N') {
        $this->service->realizaCommit();
      } else {
        $this->service->realizaRollback();
      }
      
      $response->addDataSet(new DataSet('data', $procedure));
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function alteracaoAutomaticaProduto(Row $request, Response $response) {
    $row = $request->getRow();
    
    $rows = $row['ROWS'];
    
    try {
      $valido = true;
      
      $this->service->beginTransaction();
      
      foreach($rows as $r) {
        $r['CDPRODRUP'] = $row['CDPRODUTO'];
        $r['CDMARCARUP'] = $row['CDMARCA'];
        $validaAlteraMigracaoOL = $this->service->validaAlteraMigracaoOL($r);
        
        if($validaAlteraMigracaoOL['V_ERROS'] != '0000000000') {
          $valido = false;
        }
      }
      
      if(!$valido) {
        $this->service->realizaRollback();
        throw new \Exception('Não foi possível concluir o processo de Alteração Automática de Produto.<br>Por favor entre em contato com o Suporte Teknisa!', 500);
      } else {
        $this->service->realizaCommit();
      }
      
      foreach($rows as $r) {
        $this->service->beginTransaction();
        
        $r['CDPRODRUP'] = $row['CDPRODUTO'];
        $r['CDMARCARUP'] = 'WWWWWWWWWW';
        $r['P_CDFILIOPERLOG'] = $row['P_CDFILIOPERLOG'];
        $r['P_NRSEQRUPTURAOL'] = $row['P_NRSEQRUPTURAOL'];
        $r['P_CDPRODINI'] = $row['P_CDPRODINI'];
        $r['P_CDPRODFIN'] = $row['P_CDPRODFIN'];
        $r['P_MENS'] = $row['P_MENS'];
        
        $procedureAlteraMigracaoOL = $this->service->procedureAlteraMigracaoOL($r);
        
        if($procedureAlteraMigracaoOL['P_ERROS'] == '0000000000' || $procedureAlteraMigracaoOL['P_ERROS'] == '0') {
          $this->service->realizaCommit();
        } else if($procedureAlteraMigracaoOL['P_ERROS'] == '0000000010' || $procedureAlteraMigracaoOL['P_ERROS'] == '10') {
          $this->service->realizaRollback();
          throw new \Exception('Não foi possível concluir o processo de Alteração Automática de Produto.<br>Produto sem cotação de Preço!', 500);
        } else if(strpos($procedureAlteraMigracaoOL['P_ERROS'], '_')) {
          $this->service->realizaRollback();
          throw new \Exception('Não foi possível concluir o processo de Alteração Automática de Produto.<br>Produto sem disponibilidade em estoque! (' + explode("_", $procedureAlteraMigracaoOL['P_ERROS'])[1] + ').', 500);
        }
      }
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function getEstoqueProduto(Row $request, Response $response) {
    $this->service->beginTransaction();
    
    try {
      $row = $request->getRow();
      
      $estoque = $this->service->getEstoqueProduto($row); 
      
      $response->addDataSet(new DataSet('estoque', $estoque));
      $this->service->realizaCommit();
    } catch (\Exception $e) {
      $this->service->realizaRollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}