<?php
namespace Folha\Logic\ESocial\Eventos\Data;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1390 - Receita de Atividades Concomitantes.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1390
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1390, Receita de Atividades
     * Concomitantes.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $idRetificacao           Identificador de Retificação dos arquivos ou não.
     * @param $nrRecibo                Número do Recibo do primeiro envio em caso de Retificação.
     * @param $idApuracao              Identificador da apuração Mensal ou Anual.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $idRetificacao, $nrRecibo, $idApuracao)
    {
        $fatorMes = null;
        $fator13 = null;

        return \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1390(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $idRetificacao, $nrRecibo, $idApuracao, $fatorMes, $fator13
        );
    }

}